/*
 * Copyright 2021 - 2022 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.springframework.sbm.build.migration.actions;

import org.springframework.sbm.build.api.BuildFile;
import org.springframework.sbm.engine.recipe.Action;
import org.springframework.sbm.engine.context.ProjectContext;
import org.springframework.sbm.project.resource.TestProjectContext;
import org.springframework.sbm.testhelper.common.utils.TestDiff;

import static org.assertj.core.api.Assertions.assertThat;

public class OpenRewriteMavenBuildFileTestSupport {

    public static void verifyRefactoring(String given, String expected, Action actionUnderTest) {
        ProjectContext projectContext = TestProjectContext.buildProjectContext().withMavenRootBuildFileSource(given).build();

        actionUnderTest.apply(projectContext);

        BuildFile buildFile = projectContext.getBuildFile();

        assertThat(buildFile.print())
                .as(TestDiff.of(buildFile.print(), expected))
                .isEqualTo(expected);
    }
}
